'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.stringToClosure = undefined;
exports.compileClosure = compileClosure;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _superagent = require('superagent');

var _superagent2 = _interopRequireDefault(_superagent);

var _superagentPromise = require('superagent-promise');

var _superagentPromise2 = _interopRequireDefault(_superagentPromise);

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

var _compiler = require('./compiler');

var _vm = require('./vm');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var readFile = (0, _bluebird.promisify)(_fs2.default.readFile);
var directory = (0, _bluebird.promisify)(_fs2.default.readdir);

var lib = {
    request: (0, _superagentPromise2.default)(_superagent2.default, _bluebird2.default),
    ramda: _ramda2.default
};

/**
* Converts a string to a native JS closure
*
* @param {String} code
* @return {*}
*/
var stringToClosure = exports.stringToClosure = function stringToClosure() {
    return (0, _vm.runInBrowser)({ module: {}, exports: {} })(_compiler.compileES6.apply(undefined, arguments));
};

/**
 * Adapts the user provided closure to Inquirer style. If the user provides
 * a 2+ arguments closure, it has a done callback; otherwise expect to be a
 * promise. If a promise is not provided, it is an error
 *
 * @param {String} name
 * @param {String} source
 * @return {(Object, String) -> Promise}
 */
function compileClosure(name, source) {
    var functor = stringToClosure(source);
    if (!(0, _ramda.is)(Function, functor)) {
        throw new TypeError('autocomplete/' + name + '.js must export a function');
    }

    return function (params, input) {
        return new _bluebird2.default(function (resolve) {
            var context = { params: params, input: input || '', lib: lib };
            if (functor.length < 2) {
                return resolve(functor(context));
            }

            functor(context, resolve);
        });
    };
}

/**
 * Reads all the autocomplete sources and returns an object containing the
 * name of the parameter as key and the source as value
 *
 * @return {Promise}
 */

exports.default = function () {
    return directory('./autocomplete/').catch({ code: 'ENOENT' }, function () {
        return [];
    }).filter((0, _ramda.test)(/^.*\.js$/)).map(function (file) {
        return (0, _bluebird.all)([(0, _ramda.take)(file.length - 3, file), readFile(_path2.default.join('autocomplete', file), 'utf-8')]);
    }).then(_ramda.fromPairs);
};
//# sourceMappingURL=autocomplete.js.map