'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.default = boilerplate;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _process = require('process');

var _process2 = _interopRequireDefault(_process);

var _bluebird = require('bluebird');

var _ramda = require('ramda');

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _superagent = require('superagent');

var _superagent2 = _interopRequireDefault(_superagent);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _input = require('./input');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var request = _superagent2.default.agent();
var createFolder = (0, _bluebird.promisify)(_fs2.default.mkdir);
var createFile = (0, _bluebird.promisify)(_fs2.default.writeFile);

/**
 * Formats a formatted String
 *
 * @param {String} source
 * @return {String}
 */
var format = function format() {
    return (0, _ramda.join)('')(function () {
        return (0, _ramda.append)('\n')(function () {
            return (0, _ramda.dropWhile)((0, _ramda.equals)('\n'))((0, _ramda.replace)(/\n {8}/g, '\n').apply(undefined, arguments));
        }.apply(undefined, arguments));
    }.apply(undefined, arguments));
};

/**
 * Returns the default name for the project according to the OS
 *
 * @param {String} directory
 * @return {String}
 */
var getDefaultName = function getDefaultName() {
    return (0, _ramda.last)((0, _ramda.cond)([[function () {
        return (0, _ramda.startsWith)('win32', _process2.default.platform);
    }, (0, _ramda.split)('\\')], [_ramda.T, (0, _ramda.split)('/')]]).apply(undefined, arguments));
};

/**
 * Generate the answers from the stdin.
 *
 * @return {Promise}
 */
function askQuestions() {
    return request.get('https://app.rung.com.br/api/categories').then(function () {
        return (0, _ramda.map)(function (_ref) {
            var name = _ref.name,
                value = _ref.alias;
            return { name: name, value: value };
        })((0, _ramda.prop)('body').apply(undefined, arguments));
    }).catch(function () {
        return [{ name: 'Miscellaneous', value: 'miscellaneous' }];
    }).then(function (categories) {
        return [{ name: 'name', message: 'Project name', default: getDefaultName(_process2.default.cwd()) }, { name: 'version', message: 'Version', default: '1.0.0', validate: function () {
                return Boolean(_semver2.default.valid.apply(_semver2.default, arguments));
            } }, { name: 'title', message: 'Title', default: 'Untitled' }, { name: 'description', message: 'Description' }, { name: 'category', type: 'list', message: 'Category', default: 'miscellaneous', choices: categories }, { name: 'license', message: 'license', default: 'MIT' }];
    }).then(_inquirer2.default.createPromptModule());
}

/**
 * Creates the folder for the boilerplate based on package name. If the folder
 * already exists, throw an error
 * Queria estar morta
 *
 * @param {Object} answers
 * @return {Promise}
 */
function createBoilerplateFolder(answers) {
    return createFolder(answers.name).tap(createFolder(answers.name + '/info')).catch(function () {
        return (0, _bluebird.reject)(new Error('Unable to create folder ' + answers.name));
    }).return(answers);
}

/**
 * Returns an object in the format { filename :: String, content :: String }
 * containing meta-informations about the file
 *
 * @param {Object} answers
 * @return {Object}
 */
function getPackageMetaFile(answers) {
    var packageFields = ['name', 'version', 'license', 'category'];
    var packageObject = (0, _ramda.merge)((0, _ramda.pick)(packageFields, answers), { dependencies: { 'rung-cli': '1.0.0' } });

    return {
        filename: _path2.default.join(answers.name, 'package.json'),
        content: (0, _stringify2.default)(packageObject, null, 2)
    };
}

/**
 * Content about README.md file
 *
 * @param {Object} answers
 * @return {Object}
 */
function getReadMeMetaFile(answers) {
    var content = format('\n        # Rung \u2500 ' + answers.title + '\n\n        # Development\n\n        - Use `yarn` to install the dependencies\n        - Use `rung run` to start the CLI wizard\n    ');

    return { filename: _path2.default.join(answers.name, 'README.md'), content: content };
}

/**
 * Content about info files
 *
 * @param {Object} answers
 * @return {Object}
 */
function getInfoFiles(answers) {
    var locales = ['en', 'es', 'pt_BR'];
    return (0, _ramda.map)(function (locale) {
        return { filename: _path2.default.join(answers.name, 'info/' + locale + '.md'), content: '' };
    })(locales);
}

/**
 * Content about index.js file
 *
 * @param {Object} answers
 * @return {Object}
 */
function getIndexFile(answers) {
    var content = format('\n        import { create } from \'rung-sdk\';\n        import { String as Text } from \'rung-cli/dist/types\';\n\n        function render(name) {\n            return <b>{ _(\'Hello {{name}}\', { name }) }</b>;\n        }\n\n        function main(context) {\n            const { name } = context.params;\n            return {\n                alerts: [{\n                    title: _(\'Welcome\'),\n                    content: render(name),\n                    resources: []\n                }]\n            };\n        }\n\n        const params = {\n            name: {\n                description: _(\'What is your name?\'),\n                type: Text\n            }\n        };\n\n        export default create(main, {\n            params,\n            primaryKey: true,\n            title: _(' + (0, _stringify2.default)(answers.title) + '),\n            description: _(' + (0, _stringify2.default)(answers.description) + '),\n            preview: render(\'Trixie\')\n        });\n    ');

    return { filename: _path2.default.join(answers.name, 'index.js'), content: content };
}

/**
 * Creates a boilerplate project
 *
 * @return {Promise}
 */
function boilerplate() {
    return askQuestions().then(createBoilerplateFolder).then((0, _ramda.juxt)([getPackageMetaFile, getReadMeMetaFile, getIndexFile, getInfoFiles])).then(_ramda.flatten).then(function () {
        return (0, _bluebird.all)((0, _ramda.map)(function (_ref2) {
            var filename = _ref2.filename,
                content = _ref2.content;
            return createFile(filename, content);
        }).apply(undefined, arguments));
    }).then(function () {
        return (0, _input.emitSuccess)('project generated');
    });
}
//# sourceMappingURL=boilerplate.js.map