'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

/**
 * Ensures there are missing no files in order to a allow a basic compilation
 * and filter the used modules. It also warns about possible improvements in the
 * apps
 *
 * @param {String[]} files
 * @return {Promise}
 */
var filterFiles = function () {
    var _ref4 = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee(files) {
        var clearModule, resources, missing, infoFiles;
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        clearModule = (0, _ramda.replace)(/^\.\//, '');
                        resources = (0, _ramda.filter)((0, _ramda.test)(/^((icon\.png)|(README(\.\w+)?\.md))$/))(files);
                        missing = (0, _ramda.without)(files, requiredFiles);

                        if (!(missing.length > 0)) {
                            _context.next = 5;
                            break;
                        }

                        return _context.abrupt('return', (0, _bluebird.reject)(Error('missing ' + missing.join(', ') + ' from the project')));

                    case 5:

                        if (!(0, _ramda.contains)('icon.png', files)) {
                            (0, _input.emitWarning)('compiling app without providing an icon.png file');
                        }

                        _context.t0 = (0, _ramda.map)(function (_2) {
                            return _path2.default.join('info/', _2);
                        });
                        _context.t1 = (0, _ramda.filter)((0, _ramda.test)(/[a-z]{2}(_[A-Z]{2,3})?\.md/));
                        _context.next = 10;
                        return (0, _bluebird.resolve)(listFiles('info'));

                    case 10:
                        _context.t2 = _context.sent;
                        _context.t3 = (0, _context.t1)(_context.t2);
                        infoFiles = (0, _context.t0)(_context.t3);
                        return _context.abrupt('return', fs.readFileAsync('index.js', 'utf-8').then(_module.inspect).then((0, _ramda.over)((0, _ramda.lensProp)('modules'), (0, _ramda.filter)((0, _ramda.startsWith)('./')))).then(function (_ref5) {
                            var code = _ref5.code,
                                modules = _ref5.modules;
                            return {
                                code: code,
                                files: (0, _ramda.union)(modules.map(clearModule), [].concat((0, _toConsumableArray3.default)(resources), requiredFiles, (0, _toConsumableArray3.default)(infoFiles)))
                            };
                        }));

                    case 14:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this);
    }));

    return function filterFiles(_x) {
        return _ref4.apply(this, arguments);
    };
}();

/**
 * Returns all the files in a directory if it exists. Otherwise, return an
 * empty array as fallback (everything inside a promise)
 *
 * @param {String} directory
 * @return {String[]}
 */


exports.default = build;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _jszip = require('jszip');

var _jszip2 = _interopRequireDefault(_jszip);

var _ramda = require('ramda');

var _deepmerge = require('deepmerge');

var _deepmerge2 = _interopRequireDefault(_deepmerge);

var _input = require('./input');

var _vm = require('./vm');

var _module = require('./module');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var fs = (0, _bluebird.promisifyAll)(require('fs'));

var defaultFileOptions = { date: new Date(1149562800000) };
var requiredFiles = ['package.json', 'index.js'];

var localeByFile = function localeByFile() {
    return (0, _ramda.takeWhile)((0, _ramda.complement)((0, _ramda.equals)('.')))((0, _ramda.drop)(8).apply(undefined, arguments));
};

/**
 * Converts a list of locale files to pairs containing locale string and content
 *
 * @param {String[]} localeFiles
 * @return {Promise}
 */
function localesToPairs(localeFiles) {
    return (0, _bluebird.all)(localeFiles.map(function (localeFile) {
        return fs.readFileAsync(localeFile, 'utf-8').then(JSON.parse).then(function (json) {
            return [localeByFile(localeFile), json];
        });
    }));
}

/**
 * Projects locale for each translatable subfield
 *
 * @param {String} locale
 * @param {Object} config
 * @return {Object}
 */
var project = (0, _ramda.curry)(function (locale, config) {
    return {
        title: (0, _defineProperty3.default)({}, locale, config.title),
        description: (0, _defineProperty3.default)({}, locale, config.description),
        preview: (0, _defineProperty3.default)({}, locale, config.preview),
        params: (0, _ramda.mapObjIndexed)(function (param) {
            return (0, _ramda.merge)(param, { description: (0, _defineProperty3.default)({}, locale, param.description) });
        }, config.params)
    };
});

/**
 * Lazily runs the app using all possible listed locales and extracts
 * the meta-data.
 *
 * @param {String} source
 * @param {[(String, *)]} locales
 * @return {Promise}
 */
var runInAllLocales = (0, _ramda.curry)(function (source, locales) {
    return (0, _module.compileModulesFromSource)(source).then(function (modules) {
        return (0, _bluebird.all)([['default', {}]].concat((0, _toConsumableArray3.default)(locales)).map(function (_ref) {
            var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
                locale = _ref2[0],
                strings = _ref2[1];

            return (0, _vm.getProperties)({ name: 'precompile-' + locale, source: source }, strings, modules).then(project(locale));
        })).then((0, _ramda.ifElse)((0, _ramda.propEq)('length', 1), _ramda.head, (0, _ramda.unary)(_deepmerge2.default.all)));
    });
});

/**
 * Creates a meta file where the information about precompilation is stored
 *
 * @param {Object} locales
 * @return {Promise}
 */
function createMetaFile(locales) {
    return fs.writeFileAsync('.meta', (0, _stringify2.default)(locales));
}

/**
 * Precompiles linked files, generating a .meta file with all the meta data
 *
 * @param {Object<String, String[]>} { code, files }
 * @return {Promise}
 */
function precompile(_ref3) {
    var code = _ref3.code,
        files = _ref3.files;

    return (0, _bluebird.resolve)(files).then((0, _ramda.filter)((0, _ramda.test)(/^locales(\/|\\)[a-z]{2,3}(_[A-Z]{2})?\.json$/))).then(localesToPairs).then(runInAllLocales(code)).then(createMetaFile).return(['.meta'].concat((0, _toConsumableArray3.default)(files)));
}function listFiles(directory) {
    return fs.lstatAsync(directory).then(function (lstat) {
        return lstat.isDirectory() ? fs.readdirAsync(directory) : [];
    }).catchReturn([]);
}

/**
 * Links autocomplete files
 *
 * @return {Promise}
 */
function linkAutoComplete() {
    return listFiles('autocomplete').then(function () {
        return (0, _ramda.map)(_3 => {
            return _path2.default.join('autocomplete', _3);
        })((0, _ramda.filter)((0, _ramda.endsWith)('.js')).apply(undefined, arguments));
    }).tap(function (files) {
        return (0, _bluebird.all)(files.map(function (file) {
            return fs.readFileAsync(file).then((0, _module.ensureNoImports)(file));
        }));
    });
}

/**
 * Links locale files
 *
 * @return {Promise}
 */
function linkLocales() {
    return listFiles('locales').then(function () {
        return (0, _ramda.map)(_4 => {
            return _path2.default.join('locales', _4);
        })((0, _ramda.filter)((0, _ramda.test)(/^[a-z]{2}(_[A-Z]{2,3})?\.json$/)).apply(undefined, arguments));
    }).filter(function (location) {
        return fs.readFileAsync(location).then(function () {
            return (0, _ramda.is)(Object)(JSON.parse.apply(JSON, arguments));
        }).catchReturn(false);
    }).catchReturn([]);
}

/**
 * Links the files to precompilation, including locales and autocomplete
 * scripts. For autocomplete files, ensuring it is a valid script without
 * requires. For locales, filtering true locale files and appending the full
 * qualified name for current files.
 *
 * @param {Object<String, String[]>} { code, files }
 * @return {Promise}
 */
function linkFiles(_ref6) {
    var code = _ref6.code,
        files = _ref6.files;

    return (0, _bluebird.all)([linkLocales(), linkAutoComplete()]).spread(_ramda.union).then(function () {
        return function (files) {
            return { code: code, files: files };
        }(function () {
            return (0, _ramda.sort)(_ramda.subtract)((0, _ramda.union)(files).apply(undefined, arguments));
        }.apply(undefined, arguments));
    });
}

/**
 * Opens package.json and extrats its contents. Returns a promise containing
 * the file list to be zipped and the package.json content parsed
 *
 * @param {String} dir
 * @return {Promise}
 */
function getProjectName(dir) {
    return fs.readFileAsync(_path2.default.join(dir, 'package.json')).then(function () {
        return (_5 => {
            return _5.name;
        })(JSON.parse(...arguments));
    }).catchThrow(new Error('Failed to parse package.json from the project'));
}

/**
 * Generates a zip package using a node buffer containing the necessary files
 *
 * @param {String} dir
 * @param {String[]} files
 * @param {String} name
 */
var createZip = (0, _ramda.curry)(function (dir, files) {
    var zip = new _jszip2.default();
    files.forEach(function (filename) {
        zip.file(filename, fs.readFileSync(_path2.default.join(dir, filename)), defaultFileOptions);
    });
    return zip;
});

/**
 * Taking account the -o parameter can be used to specify the output directory,
 * let's deal with it
 *
 * @param {String} customPath
 * @param {String} filename
 * @return {String}
 */
function resolveOutputTarget(customPath, filename) {
    var realPath = _path2.default.resolve('.', customPath);

    var getPath = (0, _ramda.tryCatch)(function (realPath) {
        return fs.lstatSync(realPath).isDirectory() ? _path2.default.join(realPath, filename) : realPath;
    }, _ramda.identity);

    return getPath(realPath);
}

/**
 * Saves the zip file from buffer to the filesystem
 *
 * @param {String} dir
 * @param {Zip} zip
 * @param {String} name
 */
var saveZip = (0, _ramda.curry)(function (dir, zip, name) {
    var target = resolveOutputTarget(dir, name + '.rung');

    return new _bluebird2.default(function (resolve, reject) {
        zip.generateNodeStream({ type: 'nodebuffer', streamFiles: true }).pipe(fs.createWriteStream(target)).on('error', reject).on('finish', function () {
            return resolve(target);
        });
    });
});

/**
 * Precompiles an app and generates a .rung package
 *
 * @param {Object} args
 */
function build(args) {
    var dir = _path2.default.resolve('.', args._[1] || '');

    return fs.readdirAsync(dir).then(filterFiles).then(linkFiles).then(precompile).then(createZip(dir)).then(function (zip) {
        return (0, _bluebird.all)([zip, getProjectName(dir)]);
    }).spread(saveZip(args.output || '.')).tap(function () {
        return (0, _input.emitSuccess)('Rung app compilation');
    });
}
//# sourceMappingURL=build.js.map