'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.compileHTML = compileHTML;
exports.compileES6 = compileES6;

var _dasherize = require('dasherize');

var _dasherize2 = _interopRequireDefault(_dasherize);

var _ramda = require('ramda');

var _babelCore = require('babel-core');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Generates CSS string from an object
 *
 * @param {Object} obj
 * @return {String}
 */
var compileCSS = function compileCSS() {
    return (0, _stringify2.default)(function () {
        return (0, _ramda.join)(';')(function () {
            return (0, _ramda.map)(function (_ref) {
                var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
                    key = _ref2[0],
                    value = _ref2[1];

                return (0, _dasherize2.default)(key) + ':' + value;
            })(_ramda.toPairs.apply(undefined, arguments));
        }.apply(undefined, arguments));
    }.apply(undefined, arguments));
};

/**
 * Generates HTML string for element properties
 *
 * @param {Object} props
 * @return {String}
 */
function compileProps(props) {
    var transformKey = (0, _ramda.when)((0, _ramda.equals)('className'), function () {
        return 'class';
    });
    var transformValue = (0, _ramda.ifElse)(function () {
        return (0, _ramda.equals)('Object')(_ramda.type.apply(undefined, arguments));
    }, compileCSS, (0, _ramda.unary)(_stringify2.default));

    var result = (0, _ramda.join)(' ')((0, _ramda.map)(function (_ref3) {
        var _ref4 = (0, _slicedToArray3.default)(_ref3, 2),
            key = _ref4[0],
            value = _ref4[1];

        return transformKey(key) + '=' + transformValue(value);
    })((0, _ramda.toPairs)(props)));

    return result.length === 0 ? '' : ' ' + result;
}

/**
 * Compiles a self-closing tag, dealing with elements that may or not be
 * self-closing
 *
 * @param {String} tag - JSX component name
 * @param {Object} props - Element properties
 */
function compileSelfClosingTag(tag, props) {
    var compiledProps = compileProps(props);

    return (0, _ramda.contains)(tag, ['br', 'hr', 'img']) ? '<' + tag + compiledProps + ' />' : '<' + tag + compiledProps + '></' + tag + '>';
}

/**
 * Generates HTML source code directly from JSX
 *
 * @param {String} tag - JSX component name
 * @param {Object} props - Element properties
 * @param {Array} children - Items to append to inner component
 * @return {String}
 */
function compileHTML(tag, props) {
    var filteredTag = (0, _ramda.when)((0, _ramda.contains)(_ramda.__, ['style', 'script']), function () {
        return 'span';
    })(tag);
    var render = (0, _ramda.when)(function () {
        return (0, _ramda.equals)('Array')(_ramda.type.apply(undefined, arguments));
    }, (0, _ramda.join)(''));

    for (var _len = arguments.length, children = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
        children[_key - 2] = arguments[_key];
    }

    return children.length === 0 ? compileSelfClosingTag(filteredTag, props) : '<' + filteredTag + compileProps(props) + '>' + children.map(render).join('') + '</' + filteredTag + '>';
}

/**
 * Precompiles ES6 source to ES5 in order to keep retrocompatibily
 *
 * @author Marcelo Haskell Camargo
 * @param {String} source
 * @param {String} property - code, map or ast
 * @return {Promise}
 */
function compileES6(source) {
    var result = (0, _babelCore.transform)(source, {
        comments: false,
        compact: true,
        presets: ['es2015', 'react'],
        plugins: [['transform-react-jsx', { pragma: '__render__' }]]
    });

    return result.code;
}
//# sourceMappingURL=compiler.js.map