'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.clear = clear;
exports.upsert = upsert;
exports.read = read;

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _bluebird = require('bluebird');

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _ramda = require('ramda');

var _prettyjson = require('prettyjson');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var rm = (0, _bluebird.promisify)(_rimraf2.default);
var createFolder = (0, _bluebird.promisify)(_fs2.default.mkdir);
var createFile = (0, _bluebird.promisify)(_fs2.default.writeFile);
var readFile = (0, _bluebird.promisify)(_fs2.default.readFile);

/**
 * Marshalls the JS input
 *
 * @param {Mixed} input
 * @return {Promise}
 */
function serialize(input) {
    var value = (0, _stringify2.default)(input);
    return value === undefined ? (0, _bluebird.reject)(new Error('Unsupported type ' + (0, _ramda.type)(input))) : (0, _bluebird.resolve)(value);
}

/**
 * Gets the location of a database file
 *
 * @param {String} name - Database name
 * @return {String}
 */
function location(name) {
    return _path2.default.join(_os2.default.homedir(), '.rung', name + '.db');
}

/**
 * Clears a database file by removing it
 *
 * @param {String} name - Database name
 * @return {Promise}
 */
function clear(name) {
    return rm(location(name));
}

/**
 * Creates or updates a database file
 *
 * @param {String} name - Database name
 * @param {Mixed} store - Content to save
 * @return {Promise}
 */
function upsert(name, store) {
    // When store is undefined, drop the file
    return store === undefined ? (0, _bluebird.resolve)() : resolveRungFolder().then(function () {
        return serialize(store);
    }).then(function (value) {
        return createFile(location(name), value);
    });
}

/**
 * Reads data from database file
 *
 * @param {String} name
 * @return {Promise}
 */
function read(name) {
    var file = location(name);
    return readFile(file, 'utf-8').then(JSON.parse).catchReturn(undefined);
}

/**
 * Creates the .rung folder when it doesn't exist
 *
 * @return {Promise}
 */
function resolveRungFolder() {
    var folder = _path2.default.join(_os2.default.homedir(), '.rung');
    var createIfNotExists = (0, _ramda.tryCatch)(function () {
        return _fs2.default.lstatSync(folder).isDirectory() ? (0, _bluebird.resolve)() : (0, _bluebird.reject)(new Error('~/.rung is not a directory'));
    }, function () {
        return createFolder(folder);
    });

    return createIfNotExists();
}

function getPackage() {
    return readFile('package.json').then(JSON.parse);
}

function cliRead() {
    return getPackage().then(function (_ref) {
        var name = _ref.name;
        return read(name);
    }).then(_prettyjson.render).tap(console.log).catch(function () {
        return (0, _bluebird.reject)(new Error('Unable to read database'));
    });
}

function cliClear() {
    return getPackage().then(function (_ref2) {
        var name = _ref2.name;
        return clear(name);
    }).catch(function () {
        return (0, _bluebird.reject)(new Error('Unable to clear database'));
    });
}

exports.default = function (_ref3) {
    var option = _ref3.option;
    return (0, _ramda.cond)([[(0, _ramda.equals)('read'), cliRead], [(0, _ramda.equals)('clear'), cliClear], [_ramda.T, function (option) {
        return (0, _bluebird.reject)(new Error('Unknown option ' + option));
    }]])(option);
};
//# sourceMappingURL=db.js.map