'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.translator = undefined;
exports.getLocale = getLocale;
exports.getLocaleStrings = getLocaleStrings;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _bluebird = require('bluebird');

var _osLocale = require('os-locale');

var _osLocale2 = _interopRequireDefault(_osLocale);

var _ramda = require('ramda');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var readFile = (0, _bluebird.promisify)(_fs2.default.readFile);

/**
 * Returns the user locale. Firstly consider the env variable and, if it
 * doesn't exist, consider machine's locale
 *
 * @return {Promise}
 */
function getLocale() {
    var RUNG_LOCALE = process.env.RUNG_LOCALE;

    return (0, _bluebird.resolve)(RUNG_LOCALE ? RUNG_LOCALE : (0, _osLocale2.default)());
}

/**
 * Translates a string or fallback to the key
 *
 * @param {Object} map - Object containing app strings
 * @param {String} key - Key to search in hashmap
 * @return {String}
 */
var translator = exports.translator = (0, _ramda.curry)(function (map, key) {
    var params = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    var sentence = (0, _ramda.propOr)(key, key, map);

    return (0, _ramda.replace)(/{{(\w+)}}/g, function (full, partial) {
        return params[partial] || full;
    }, sentence);
});

/**
 * Reads the JSON file corresponding to a specific locale
 *
 * @return {Promise}
 */
function getLocaleStrings() {
    return getLocale().then(function (locale) {
        return readFile(_path2.default.join('locales', locale + '.json'));
    }).then(JSON.parse).catchReturn({});
}
//# sourceMappingURL=i18n.js.map