'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.components = exports.emitSuccess = exports.emitError = exports.emitWarning = exports.emitInfo = undefined;

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

exports.ask = ask;

var _process = require('process');

var _process2 = _interopRequireDefault(_process);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _bluebird = require('bluebird');

var _ramda = require('ramda');

var _safe = require('colors/safe');

var _inquirer = require('inquirer');

var _types = require('./types');

var _autocomplete = require('./autocomplete');

var _autocomplete2 = _interopRequireDefault(_autocomplete);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Emits an info message to stdout
 *
 * @param {String} message
 * @return {Promise}
 */
var emitInfo = exports.emitInfo = function emitInfo() {
    return (0, _bluebird.resolve)(function () {
        return console.log(function () {
            return (0, _safe.cyan)((0, _ramda.concat)(' ℹ Info: ').apply(undefined, arguments));
        }.apply(undefined, arguments));
    }.apply(undefined, arguments));
};

/**
 * Emits a warning to stdout
 *
 * @param {String} message
 * @return {Promise}
 */
var emitWarning = exports.emitWarning = function emitWarning() {
    return (0, _bluebird.resolve)(function () {
        return console.log(function () {
            return (0, _safe.yellow)((0, _ramda.concat)(' ⚠ Warning: ').apply(undefined, arguments));
        }.apply(undefined, arguments));
    }.apply(undefined, arguments));
};

/**
 * Emits an error to stdout
 *
 * @param {String} message
 * @return {Promise}
 */
var emitError = exports.emitError = function emitError() {
    return (0, _bluebird.resolve)(function () {
        return console.log(function () {
            return (0, _safe.red)((0, _ramda.concat)(' ✗ Error: ').apply(undefined, arguments));
        }.apply(undefined, arguments));
    }.apply(undefined, arguments));
};

/**
 * Emits a success message
 *
 * @param {String} message
 * @return {Promise}
 */
var emitSuccess = exports.emitSuccess = function emitSuccess() {
    return (0, _bluebird.resolve)(function () {
        return console.log(function () {
            return (0, _safe.green)((0, _ramda.concat)(' ✔ Success: ').apply(undefined, arguments));
        }.apply(undefined, arguments));
    }.apply(undefined, arguments));
};

/**
 * Renames the keys of an object
 *
 * @sig {a: b} -> {a: *} -> {b: *}
 */
var renameKeys = (0, _ramda.curry)(function (keysMap, obj) {
    return (0, _ramda.reduce)(function (acc, key) {
        return (0, _ramda.assoc)(keysMap[key] || key, obj[key], acc);
    }, {}, (0, _ramda.keys)(obj));
});

var objectToChoices = function objectToChoices() {
    return (0, _ramda.map)(function (_ref) {
        var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
            value = _ref2[0],
            name = _ref2[1];

        return { name: name, value: value };
    })(_ramda.toPairs.apply(undefined, arguments));
};

var components = exports.components = {
    Calendar: function () {
        return {
            type: 'datetime',
            format: ['m', '/', 'd', '/', 'yy'],
            filter: _types.filter.Calendar };
    },
    Char: function Char(_ref3) {
        var length = _ref3.length;
        return { filter: _types.filter.Char(length) };
    },
    Checkbox: function () {
        return { type: 'confirm' };
    },
    Color: function () {
        return {
            type: 'chalk-pipe',
            validate: _types.validator.Color };
    },
    DoubleRange: function DoubleRange(_ref4) {
        var from = _ref4.from,
            to = _ref4.to;
        return {
            filter: _types.filter.Double,
            validate: _types.validator.Range(from, to) };
    },
    DateTime: function () {
        return { type: 'datetime' };
    },
    Double: function () {
        return { validate: _types.validator.Double, filter: _types.filter.Double };
    },
    Email: function () {
        return { validate: _types.validator.Email };
    },
    Integer: function () {
        return { validate: _types.validator.Integer, filter: _types.filter.Integer };
    },
    IntegerRange: function IntegerRange(_ref5) {
        var from = _ref5.from,
            to = _ref5.to;
        return {
            filter: _types.filter.Integer,
            validate: _types.validator.Range(from, to) };
    },
    IntegerMultiRange: function IntegerMultiRange(_ref6) {
        var from = _ref6.from,
            to = _ref6.to;
        return {
            filter: _types.filter.IntegerMultiRange,
            validate: _types.validator.IntegerMultiRange(from, to) };
    },
    Natural: function () {
        return { validate: _types.validator.Natural, filter: _types.filter.Integer };
    },
    OneOf: function OneOf(_ref7) {
        var values = _ref7.values;
        return {
            type: 'list',
            choices: values,
            validate: _types.validator.OneOf(values) };
    },
    String: function () {
        return { type: 'input' };
    },
    Url: function () {
        return { validate: _types.validator.Url };
    },
    Money: function () {
        return { validate: _types.validator.Money, filter: _types.filter.Money };
    },
    SelectBox: function SelectBox(_ref8) {
        var values = _ref8.values;
        return {
            type: 'list',
            choices: objectToChoices(values) };
    },
    MultiSelectBox: function MultiSelectBox(_ref9) {
        var values = _ref9.values;
        return {
            type: 'checkbox',
            choices: objectToChoices(values) };
    },
    File: function () {
        return { type: 'filePath', basePath: _process2.default.cwd() };
    }
};

/**
 * Custom autocomplete component
 *
 * @param {String} name
 * @param {String} source
 * @return {Function}
 */
var getAutocompleteComponent = function getAutocompleteComponent(name, source) {
    if (!source) {
        throw new Error('aren\'t you missing \'autocomplete/' + name + '.js\'?');
    }

    return function () {
        return { type: 'autocomplete', source: (0, _autocomplete.compileClosure)(name, source) };
    };
};

/**
 * Converts a Rung CLI question object to an Inquirer question object
 *
 * @author Marcelo Haskell Camargo
 * @param {String[]} sources
 * @param {String} name
 * @param {Object} config
 * @return {Object}
 */
var toInquirerQuestion = (0, _ramda.curry)(function (sources, _ref10) {
    var _ref11 = (0, _slicedToArray3.default)(_ref10, 2),
        name = _ref11[0],
        config = _ref11[1];

    var component = (0, _ramda.cond)([[function () {
        return config.type.name === 'AutoComplete';
    }, function () {
        return getAutocompleteComponent(name, sources[name]);
    }], [(0, _ramda.has)(config.type.name), _2 => {
        return _2[config.type.name];
    }], [_ramda.T, _3 => {
        return _3.String;
    }]])(components);

    return (0, _ramda.merge)((0, _ramda.merge)(_ramda.__, { name: name })(renameKeys({ description: 'message' })(config)), component(config.type));
});

var readFile = (0, _bluebird.promisify)(_fs2.default.readFile);

/**
 * Opens the provided files and returns them as node buffers
 *
 * @param {String[]} fields
 * @param {Object} answers
 * @return {Promise}
 */
var openFiles = function openFiles(fields) {
    return function () {
        return (0, _bluebird.props)((0, _ramda.mapObjIndexed)(function (value, param) {
            return (0, _ramda.when)(function () {
                return (0, _ramda.contains)(param, fields);
            }, function () {
                return readFile((0, _ramda.concat)(_process2.default.cwd() + '/').apply(undefined, arguments));
            })(value);
        }).apply(undefined, arguments));
    };
};

/**
 * Returns the pure JS values from received questions that will be answered
 *
 * @author Marcelo Haskell Camargo
 * @param {Object} questions
 * @return {Promise} answers for the questions by key
 */
function ask(questions) {
    var DatePickerPrompt = require('inquirer-datepicker-prompt');
    var ChalkPipe = require('inquirer-chalk-pipe');
    var AutocompletePrompt = require('inquirer-autocomplete-prompt');
    var FilePath = require('inquirer-file-path');
    var fileFields = (0, _ramda.keys)((0, _ramda.filter)((0, _ramda.pathEq)(['type', 'name'], 'File'))(questions));

    var prompt = (0, _inquirer.createPromptModule)();
    prompt.registerPrompt('datetime', DatePickerPrompt);
    prompt.registerPrompt('chalk-pipe', ChalkPipe);
    prompt.registerPrompt('autocomplete', AutocompletePrompt);
    prompt.registerPrompt('filePath', FilePath);
    return (0, _autocomplete2.default)().then(function (autocompleteSources) {
        return prompt((0, _ramda.map)(toInquirerQuestion(autocompleteSources))((0, _ramda.toPairs)(questions)));
    }).then(openFiles(fileFields));
}
//# sourceMappingURL=input.js.map