'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _process = require('process');

var _process2 = _interopRequireDefault(_process);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _opn = require('opn');

var _opn2 = _interopRequireDefault(_opn);

var _nodeWatch = require('node-watch');

var _nodeWatch2 = _interopRequireDefault(_nodeWatch);

var _socket = require('socket.io');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _ramda = require('ramda');

var _showdown = require('showdown');

var _run = require('./run');

var _input = require('./input');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var readDirectory = (0, _bluebird.promisify)(_fs2.default.readdir);

/**
 * Emits the Rung emoji to the live server ;)
 *
 * @return {Promise}
 */
function emitRungEmoji() {
    return (0, _input.emitInfo)((0, _ramda.join)('\n')(['', '   ___       _  _______', '  / _ \\__ __/ |/ / ___/', ' / , _/ // /    / (_ /', '/_/|_|\\_,_/_/|_/\\___/', '']));
}

/**
 * Returns an object with resource path and buffer
 *
 * @return {Promise}
 */
var getResources = function getResources() {
    var resources = _path2.default.join(__dirname, '../resources/live');
    return readDirectory(resources).map(function (filename) {
        return (0, _bluebird.props)((0, _defineProperty3.default)({}, '/' + filename, (0, _run.readFile)(_path2.default.join(resources, filename))));
    }).then(_ramda.mergeAll);
};

/**
 * Compiles the content of the cards to be compatible with HTML
 *
 * @param {Object} alerts
 * @return {Object[]}
 */
var compileMarkdown = function compileMarkdown(alerts) {
    var converter = new _showdown.Converter();
    return (0, _ramda.map)((0, _ramda.when)((0, _ramda.has)('comment'), (0, _ramda.over)((0, _ramda.lensProp)('comment'), function () {
        return converter.makeHtml((0, _ramda.replace)(/^[ \t]+/gm, '').apply(undefined, arguments));
    })))(alerts);
};

/**
 * Watches folder for file changes, hot recompiling everything and notifying
 * the clients
 *
 * @param {SocketIO} io
 * @param {Object} params
 * @return {Object}
 */
function watchChanges(io, params) {
    var folder = _process2.default.cwd();
    return (0, _nodeWatch2.default)(folder, { recursive: true }, function () {
        (0, _input.emitInfo)('changes detected. Recompiling...');
        io.sockets.emit('load');
        var start = new Date().getTime();
        (0, _run.executeWithParams)(params).tap(function (alerts) {
            var ellapsed = new Date().getTime() - start;
            (0, _input.emitSuccess)('wow! recompiled and executed in ' + ellapsed + 'ms!');
            io.sockets.emit('update', compileMarkdown(alerts));
        }).catch(function (err) {
            (0, _input.emitError)('hot compilation error, baby: ' + err.message);
            io.sockets.emit('failure', err.stack);
        });
    });
}

/**
 * Starts the stream server using sockets
 *
 * @param {Object} alerts
 * @param {Object} params
 * @param {Number} port
 * @param {Object} resources
 * @return {Promise}
 */
function startServer(alerts, params, port, resources) {
    var compiledAlerts = compileMarkdown(alerts);
    var app = _http2.default.createServer(function (req, res) {
        return res.end(resources[req.url] || resources['/index.html']);
    });
    var io = (0, _socket.listen)(app);
    io.on('connection', function (socket) {
        (0, _input.emitInfo)('new session for ' + socket.handshake.address);
        socket.emit('update', compiledAlerts);
        socket.on('disconnect', function () {
            (0, _input.emitInfo)('disconnected session ' + socket.handshake.address);
        });
    });

    return new _bluebird2.default(function (resolve) {
        return app.listen(port, function () {
            return resolve(emitRungEmoji.apply(undefined, arguments));
        });
    }).then(function () {
        return watchChanges(io, params);
    });
}

/**
 * Generates a HTML file compiled from template showing the cards as they will
 * be rendered on Rung and opens it in the default browser
 *
 * @return {Promise}
 */

exports.default = function (alerts, params) {
    return getResources().tap(_2 => {
        return startServer(alerts, params, 5001, _2);
    }).tap(function () {
        try {
            (0, _opn2.default)('http://localhost:5001/');
        } catch (err) {
            // Skip
        }
    });
};
//# sourceMappingURL=live.js.map