'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.default = publish;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _bluebird = require('bluebird');

var _ramda = require('ramda');

var _superagent = require('superagent');

var _superagent2 = _interopRequireDefault(_superagent);

var _validator = require('validator');

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _input = require('./input');

var _build = require('./build');

var _build2 = _interopRequireDefault(_build);

var _types = require('./types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var request = _superagent2.default.agent();
var rm = (0, _bluebird.promisify)(_rimraf2.default);

/**
 * Returns the Rung API URL. Emits a warning when the URL is possibly invalid
 *
 * @return {String}
 */
function fetchRungApi() {
    var envApi = process.env.RUNG_API;

    if ((0, _ramda.isNil)(envApi)) {
        return 'https://app.rung.com.br/api';
    }

    if (!(0, _validator.isURL)(envApi)) {
        (0, _input.emitWarning)('invalid API for Rung: ' + (0, _stringify2.default)(envApi));
    }

    return envApi;
}

/**
 * Publishes a Rung package from a file
 *
 * @param {String} api
 * @param {String} filename
 * @return {Promise}
 */
var publishFile = (0, _ramda.curry)(function (api, filename) {
    return request.post(api + '/metaExtensions/drafts').attach('metaExtension', filename).then(function () {
        return rm(filename);
    });
});

/**
 * Builds or uses the passed file to publication
 *
 * @param {Object} args
 * @return {Promise}
 */
function resolveInputFile(args) {
    var file = args.file;

    return file ? (0, _bluebird.resolve)(_path2.default.resolve(file)) : (0, _build2.default)(args);
}

/**
 * Authenticates and publishes the app
 *
 * @param {Object} args
 * @return {Promise}
 */
function publish(args) {
    var api = fetchRungApi();

    return _inquirer2.default.prompt([{ name: 'email', message: 'Rung email', validate: _types.validator.Email }, { name: 'password', type: 'password', message: 'Rung password' }]).then(function (payload) {
        return request.post(api + '/login').send(payload);
    }).then(function () {
        return resolveInputFile(args);
    }).then(publishFile(api)).catch(function (err) {
        (0, _input.emitError)(err.message);
    });
}
//# sourceMappingURL=publish.js.map