'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = readme;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _bluebird = require('bluebird');

var _handlebars = require('handlebars');

var _ramda = require('ramda');

var _package = require('../package');

var _vm = require('./vm');

var _run = require('./run');

var _types = require('./types');

var _compiler = require('./compiler');

var _input = require('./input');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createFile = (0, _bluebird.promisify)(_fs2.default.writeFile);

/**
 * Returns the source Handlebars template as string
 *
 * @return {Promise}
 */
function getHandlebarsTemplate() {
    return (0, _run.readFile)(_path2.default.join(__dirname, '../templates/readme.hb'), 'utf-8').then(_handlebars.compile);
}

/**
 * Converts an object to an array containing a list of { name, version }
 *
 * @param {Object} dependencies
 * @return {Array}
 */
function dependenciesToArray(dependencies) {
    return (0, _ramda.zipWith)(function (name, version) {
        return { name: name, version: version };
    }, (0, _ramda.keys)(dependencies), (0, _ramda.values)(dependencies));
}

/**
 * Converts an object to an array containing a list of { name, type, description }
 *
 * @param {Object} parameters
 * @return {Array}
 */
function parametersToArray(parameters) {
    return (0, _ramda.zipWith)(function (name, _ref) {
        var type = _ref.type,
            description = _ref.description;
        return {
            name: name,
            type: (0, _types.getTypeName)(type),
            description: description };
    }, (0, _ramda.keys)(parameters), (0, _ramda.values)(parameters));
}

/**
 * Generates a full README in Markdown with documentation about input parameters
 * and the business rules of the app
 *
 * @return {Promise}
 */
function readme() {
    return (0, _run.readFile)('package.json', 'utf-8').then(JSON.parse).then(function (_ref2) {
        var name = _ref2.name,
            version = _ref2.version,
            author = _ref2.author,
            dependencies = _ref2.dependencies;
        return (0, _bluebird.all)([{
            rungCliVersion: _package.version,
            name: name,
            version: version,
            author: author,
            escapedName: (0, _ramda.replace)(/-/g, '--', name),
            dependencies: dependenciesToArray(dependencies) }, (0, _run.compileSources)().spread(function (source, modules) {
            return (0, _vm.getProperties)({ name: 'pre-compile', source: (0, _compiler.compileES6)(source) }, {}, modules);
        })]);
    }).spread(function (partialContext, result) {
        return (0, _ramda.merge)(partialContext, {
            parameters: parametersToArray(result.params),
            description: result.description,
            title: result.title
        });
    }).then(function (context) {
        return (0, _bluebird.all)([context, getHandlebarsTemplate()]);
    }).spread(function (context, generateReadme) {
        return generateReadme(context);
    }).then(_2 => {
        return createFile('README.md', _2);
    }).tap(function () {
        return (0, _input.emitSuccess)('generated README.md');
    });
}
//# sourceMappingURL=readme.js.map