'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.executeWithParams = exports.readFile = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.compileSources = compileSources;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _bluebird = require('bluebird');

var _ramda = require('ramda');

var _cliTable = require('cli-table');

var _cliTable2 = _interopRequireDefault(_cliTable);

var _vm = require('./vm');

var _input = require('./input');

var _compiler = require('./compiler');

var _db = require('./db');

var _i18n = require('./i18n');

var _module = require('./module');

var _live = require('./live');

var _live2 = _interopRequireDefault(_live);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var percentOf = (0, _ramda.curry)(function (value, percent) {
    return value / 100 * percent;
});

var readFile = exports.readFile = (0, _bluebird.promisify)(_fs2.default.readFile);

function tableView(alerts) {
    var valuesFrom = function valuesFrom() {
        return (0, _ramda.values)((0, _ramda.mapObjIndexed)(function (_ref, key) {
            var title = _ref.title,
                _ref$content = _ref.content,
                content = _ref$content === undefined ? '' : _ref$content,
                _ref$comment = _ref.comment,
                comment = _ref$comment === undefined ? '' : _ref$comment;
            return [key, title, content, comment];
        }).apply(undefined, arguments));
    };

    var table = new _cliTable2.default({
        head: ['Key', 'Title', 'Content', 'Comment'],
        colWidths: [10, 20, 35, 26].map(function () {
            return Math.round(percentOf(process.stdout.columns || 100).apply(undefined, arguments));
        })
    });

    table.push.apply(table, (0, _toConsumableArray3.default)(valuesFrom(alerts)));
    return table.toString();
}

function compileSources() {
    return readFile('index.js', 'utf-8').then(function (index) {
        return (0, _bluebird.all)([(0, _compiler.compileES6)(index), (0, _module.compileModulesFromSource)(index)]);
    });
}

/**
 * Executes a function with the provided parameters
 *
 * @param {Object} params
 */
var executeWithParams = exports.executeWithParams = function executeWithParams(params) {
    return readFile('package.json', 'utf-8').then(JSON.parse).then(function (_ref2) {
        var name = _ref2.name;
        return (0, _bluebird.all)([name, (0, _db.read)(name), (0, _i18n.getLocaleStrings)(), (0, _i18n.getLocale)()]);
    }).spread(function (name, db, strings, locale) {
        return compileSources().spread(function (source, modules) {
            return (0, _vm.runAndGetAlerts)({ name: name, source: source }, { params: params, db: db, locale: locale }, strings, modules);
        });
    }).get('alerts');
};

exports.default = function (args) {
    return readFile('package.json', 'utf-8').then(JSON.parse).then(function (_ref3) {
        var name = _ref3.name;
        return (0, _bluebird.all)([name, (0, _db.read)(name), (0, _i18n.getLocaleStrings)(), (0, _i18n.getLocale)()]);
    }).spread(function (name, db, strings, locale) {
        return compileSources().spread(function (source, modules) {
            return (0, _vm.getProperties)({ name: name, source: source }, strings, modules).then(function () {
                return (0, _input.ask)((0, _ramda.prop)('params').apply(undefined, arguments));
            }).then(_ramda.mergeAll).then(function (params) {
                return (0, _bluebird.all)([(0, _vm.runAndGetAlerts)({ name: name, source: source }, { params: params, db: db, locale: locale }, strings, modules), params]);
            });
        });
    }).spread(function (_ref4, params) {
        var alerts = _ref4.alerts;

        if (args.live) {
            return (0, _live2.default)(alerts, params);
        }
        console.log(args.raw ? alerts : tableView(alerts));
    });
};
//# sourceMappingURL=run.js.map