'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _toArray2 = require('babel-runtime/helpers/toArray');

var _toArray3 = _interopRequireDefault(_toArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _bluebird = require('bluebird');

/**
 * Compiles the app files and returns an instance of a V8 object.
 *
 * @return {Promise<Object => Promise>}
 */
var compileApp = function () {
    var _ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee() {
        var _ref2, name, _ref3, _ref4, source, modules, strings;

        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        _context.t0 = JSON;
                        _context.next = 3;
                        return (0, _bluebird.resolve)(readFile('package.json', 'utf-8'));

                    case 3:
                        _context.t1 = _context.sent;
                        _ref2 = _context.t0.parse.call(_context.t0, _context.t1);
                        name = _ref2.name;
                        _context.next = 8;
                        return (0, _bluebird.resolve)((0, _run.compileSources)());

                    case 8:
                        _ref3 = _context.sent;
                        _ref4 = (0, _slicedToArray3.default)(_ref3, 2);
                        source = _ref4[0];
                        modules = _ref4[1];
                        _context.next = 14;
                        return (0, _bluebird.resolve)((0, _i18n.getLocaleStrings)());

                    case 14:
                        strings = _context.sent;
                        return _context.abrupt('return', function (context) {
                            return (0, _vm.runAndGetAlerts)({ name: name, source: source }, context, strings, modules);
                        });

                    case 16:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this);
    }));

    return function compileApp() {
        return _ref.apply(this, arguments);
    };
}();

/**
 * Compiles the test source.
 *
 * @return {Promise}
 */


var compileTest = function () {
    var _ref5 = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee2() {
        var source, localTestModules;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        _context2.t0 = _compiler.compileES6;
                        _context2.next = 3;
                        return (0, _bluebird.resolve)(readFile('test/index.js', 'utf-8').catchThrow(new Error('no tests provided')));

                    case 3:
                        _context2.t1 = _context2.sent;
                        source = (0, _context2.t0)(_context2.t1);
                        localTestModules = (0, _ramda.filter)((0, _ramda.either)((0, _ramda.startsWith)('./'), (0, _ramda.startsWith)('../')))((0, _module.inspect)(source).modules);

                        if (!(localTestModules.length > 0)) {
                            _context2.next = 8;
                            break;
                        }

                        return _context2.abrupt('return', (0, _bluebird.reject)(new Error('only external modules can be required in testsuite. Found ' + localTestModules.join(', '))));

                    case 8:
                        return _context2.abrupt('return', source);

                    case 9:
                    case 'end':
                        return _context2.stop();
                }
            }
        }, _callee2, this);
    }));

    return function compileTest() {
        return _ref5.apply(this, arguments);
    };
}();

/**
 * Executes the tests and returns a promise with computed results.
 * @param {Object => Promise} runWithContext - Pre-compiled application
 * @param {(String, Function)[]} tests - The test cases
 * @return {Promise}
 */


var runTests = function () {
    var _ref6 = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee3(runWithContext, tests) {
        var loop;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        if (!(tests.length === 0)) {
                            _context3.next = 2;
                            break;
                        }

                        return _context3.abrupt('return', (0, _input.emitInfo)('no tests to run'));

                    case 2:
                        loop = function loop(_ref7) {
                            var _ref8 = (0, _toArray3.default)(_ref7),
                                test = _ref8[0],
                                rest = _ref8.slice(1);

                            var report = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { passing: 0, failing: 0 };

                            if (!test) {
                                var message = report.passing + ' passing, ' + report.failing + ' failing';
                                if (report.failing > 0) {
                                    return (0, _bluebird.reject)(new Error(message));
                                }

                                return (0, _input.emitSuccess)(message);
                            }

                            var _test = (0, _slicedToArray3.default)(test, 2),
                                description = _test[0],
                                implementation = _test[1];

                            return function (future) {
                                return future.then(function () {
                                    return (0, _input.emitSuccess)(description).return((0, _ramda.evolve)({ passing: _ramda.inc })(report));
                                }).catch(function (err) {
                                    return (0, _input.emitError)(description + ':\n' + err.stack + '\n').return((0, _ramda.evolve)({ failing: _ramda.inc })(report));
                                }).then(function (_2) {
                                    return loop(rest, _2);
                                });
                            }((0, _ramda.tryCatch)(function () {
                                return (0, _bluebird.resolve)(implementation.apply(undefined, arguments));
                            }, _bluebird.reject)(runWithContext));
                        };

                        _context3.next = 5;
                        return (0, _bluebird.resolve)((0, _input.emitInfo)(tests.length + ' test case(s) found'));

                    case 5:
                        return _context3.abrupt('return', loop(tests));

                    case 6:
                    case 'end':
                        return _context3.stop();
                }
            }
        }, _callee3, this);
    }));

    return function runTests(_x, _x2) {
        return _ref6.apply(this, arguments);
    };
}();

/**
 * If possible, run the tests. Compile the tests to ES5 and the app sources to
 * V8 objects, then run the tests in the VM linking the compiled object to it.
 *
 * @return {Promise}
 */


var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _ramda = require('ramda');

var _compiler = require('./compiler');

var _i18n = require('./i18n');

var _input = require('./input');

var _module = require('./module');

var _run = require('./run');

var _vm = require('./vm');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var readFile = (0, _bluebird.promisify)(_fs2.default.readFile);exports.default = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee4() {
    var _ref10, _ref11, source, runWithContext, tests, vm, test;

    return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
            switch (_context4.prev = _context4.next) {
                case 0:
                    _context4.next = 2;
                    return (0, _bluebird.resolve)((0, _bluebird.all)([compileTest(), compileApp()]));

                case 2:
                    _ref10 = _context4.sent;
                    _ref11 = (0, _slicedToArray3.default)(_ref10, 2);
                    source = _ref11[0];
                    runWithContext = _ref11[1];
                    tests = [];
                    vm = (0, _vm.createVM)();

                    test = function test() {
                        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                            args[_key] = arguments[_key];
                        }

                        return tests.push(args);
                    };

                    vm.freeze(test, 'test');
                    vm.run(source, 'test/index.js');
                    return _context4.abrupt('return', runTests(runWithContext, tests));

                case 12:
                case 'end':
                    return _context4.stop();
            }
        }
    }, _callee4, undefined);
}));
//# sourceMappingURL=test.js.map