'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.filter = exports.validate = exports.validator = exports.getTypeName = exports.File = exports.MultiSelectBox = exports.SelectBox = exports.Location = exports.AutoComplete = exports.Calendar = exports.IntegerMultiRange = exports.Url = exports.OneOf = exports.Checkbox = exports.Email = exports.Color = exports.String = exports.Money = exports.DoubleRange = exports.IntegerRange = exports.Char = exports.Natural = exports.DateTime = exports.Double = exports.Integer = undefined;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _ramda = require('ramda');

var _validator = require('validator');

var _input = require('./input');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Integer = exports.Integer = { name: 'Integer' };
var Double = exports.Double = { name: 'Double' };
var DateTime = exports.DateTime = { name: 'DateTime' };
var Natural = exports.Natural = { name: 'Natural' };
var Char = exports.Char = function Char(length) {
    return { name: 'Char', length: length };
};
var IntegerRange = exports.IntegerRange = function IntegerRange(from, to) {
    return { name: 'IntegerRange', from: from, to: to };
};
var DoubleRange = exports.DoubleRange = function DoubleRange(from, to) {
    return { name: 'DoubleRange', from: from, to: to };
};
var Money = exports.Money = { name: 'Money' };
var String = exports.String = { name: 'String' };
var Color = exports.Color = { name: 'Color' };
var Email = exports.Email = { name: 'Email' };
var Checkbox = exports.Checkbox = { name: 'Checkbox' };
var OneOf = exports.OneOf = function OneOf(values) {
    return { name: 'OneOf', values: values };
};
var Url = exports.Url = { name: 'Url' };
var IntegerMultiRange = exports.IntegerMultiRange = function IntegerMultiRange(from, to) {
    return { name: 'IntegerMultiRange', from: from, to: to };
};
var Calendar = exports.Calendar = { name: 'Calendar' };
var AutoComplete = exports.AutoComplete = { name: 'AutoComplete' };
var Location = exports.Location = { name: 'Location' };
var SelectBox = exports.SelectBox = function SelectBox(values) {
    return { name: 'SelectBox', values: values };
};
var MultiSelectBox = exports.MultiSelectBox = function MultiSelectBox(values) {
    return { name: 'MultiSelectBox', values: values };
};
var File = exports.File = { name: 'File' };

/**
 * Returns the human-readable name of a type
 *
 * @author Marcelo Haskell Camargo
 * @param {String} type {anonymous}
 * @return {String} {anonymous}
 */
var getTypeName = exports.getTypeName = (0, _ramda.cond)([[(0, _ramda.propEq)('name', 'Char'), function (t) {
    return 'Char(' + t.length + ')';
}], [(0, _ramda.propEq)('name', 'IntegerRange'), function (t) {
    return 'IntegerRange(' + t.from + ', ' + t.to + ')';
}], [(0, _ramda.propEq)('name', 'DoubleRange'), function (t) {
    return 'DoubleRange(' + t.from + ', ' + t.to + ')';
}], [(0, _ramda.propEq)('name', 'OneOf'), function (t) {
    return 'OneOf([' + t.values.join(', ') + '])';
}], [(0, _ramda.propEq)('name', 'SelectBox'), function (t) {
    return 'SelectBox(' + (0, _stringify2.default)(t.values) + ')';
}], [(0, _ramda.propEq)('name', 'IntegerMultiRange'), function (t) {
    return 'IntegerMultiRange(' + t.from + ', ' + t.to + ')';
}], [(0, _ramda.propEq)('name', 'MultiSelectBox'), function (t) {
    return 'MultiSelectBox(' + (0, _stringify2.default)(t.values) + ')';
}], [_ramda.T, _2 => {
    return _2.name;
}]]);

var validator = exports.validator = {
    Color: _validator.isHexColor,
    Double: (0, _ramda.complement)(isNaN),
    Email: (0, _ramda.unary)(_validator.isEmail),
    Integer: (0, _ramda.complement)(isNaN),
    IntegerMultiRange: function IntegerMultiRange(from, to) {
        return (0, _ramda.allPass)([function () {
            return (0, _ramda.equals)(2)(_ramda.length.apply(undefined, arguments));
        }, (0, _ramda.none)(isNaN), function () {
            return (0, _ramda.all)(_ramda.identity)(function () {
                return (0, _ramda.values)((0, _ramda.evolve)([(0, _ramda.gte)(_ramda.__, from), (0, _ramda.lte)(_ramda.__, to)]).apply(undefined, arguments));
            }.apply(undefined, arguments));
        }]);
    },
    Money: (0, _ramda.complement)(isNaN),
    Natural: (0, _ramda.lte)(0),
    OneOf: (0, _ramda.flip)(_ramda.contains),
    Range: function Range(from, to) {
        return _3 => {
            return (0, _ramda.clamp)(from, to, _3) === _3;
        };
    },
    Url: (0, _ramda.unary)(_validator.isURL)
};

var validate = exports.validate = (0, _ramda.curry)(function (type, value) {
    return (0, _ramda.applyTo)(value)((0, _ramda.propOr)(function () {
        return true;
    }, 'validate')((0, _ramda.applyTo)(type)((0, _ramda.propOr)(function () {
        return {};
    }, type.name, _input.components))));
});

var filter = exports.filter = {
    Calendar: function Calendar(date) {
        return new Date(date);
    },
    Char: _ramda.take,
    Double: parseFloat,
    Integer: _4 => {
        return parseInt(_4, 10);
    },
    IntegerMultiRange: (0, _ramda.unless)((0, _ramda.is)(Array), function () {
        return (0, _ramda.map)(_5 => {
            return parseInt(_5, 10);
        })((0, _ramda.split)(' ').apply(undefined, arguments));
    }),
    Money: (0, _ramda.tryCatch)(function () {
        return parseFloat((0, _ramda.replace)(',', '.').apply(undefined, arguments));
    }, function () {
        return NaN;
    })
};
//# sourceMappingURL=types.js.map